@@ -96,6 +96,24 @@ enum OggzFlushOpts {
   OGGZ_FLUSH_AFTER  = 0x02
 };
 
+/**
+ * Definition of stream content types
+ */
+typedef enum OggzStreamContent {
+  OGGZ_CONTENT_THEORA = 0,
+  OGGZ_CONTENT_VORBIS,
+  OGGZ_CONTENT_SPEEX,
+  OGGZ_CONTENT_PCM,
+  OGGZ_CONTENT_CMML,
+  OGGZ_CONTENT_ANX2,
+  OGGZ_CONTENT_SKELETON,
+  OGGZ_CONTENT_FLAC0,
+  OGGZ_CONTENT_FLAC,
+  OGGZ_CONTENT_ANXDATA,
+  OGGZ_CONTENT_CELT,
+  OGGZ_CONTENT_UNKNOWN
+} OggzStreamContent;
+
 /**
  * Definitions of error return values
  */
@@ -164,6 +182,10 @@ enum OggzError {
   /** Packet disallowed due to invalid packetno */
   OGGZ_ERR_BAD_PACKETNO                 = -25,
 
+  /** Comment violates VorbisComment restrictions */
+  /* 129 == 0x81 is the frame marker for Theora's comments page ;-) */
+  OGGZ_ERR_COMMENT_INVALID              = -129,
+
   /** Guard provided by user has non-zero value */
   OGGZ_ERR_BAD_GUARD                    = -210,
 
