@@ -37,7 +37,7 @@
  * Seeking within files
  */
 
-/** \defgroup seek_api OGGZ Seek API
+/** \defgroup seek_api Oggz Seek API
  *
  * Oggz can seek on multitrack, multicodec bitstreams.
  *
@@ -51,7 +51,8 @@
  * and <a href="http://www.annodex.net/">CMML</a>.
  * Oggz is also compatible with 
  * <a href="http://www.annodex.net/">Annodex</a> streams, and supports seeking
- * on all tracks described in an Ogg Skeleton track.
+ * on all tracks described in an
+ * <a href="http://wiki.xiph.org/OggSkeleton">Ogg Skeleton</a> track.
  *
  * You need to open the file with the OGGZ_AUTO flag set:
  *
@@ -105,6 +106,17 @@ ogg_int64_t oggz_tell_units (OGGZ * oggz);
  */
 ogg_int64_t oggz_seek_units (OGGZ * oggz, ogg_int64_t units, int whence);
 
+/**
+ * Provide the exact stored granulepos (from the page header) if relevant to
+ * the current packet, or a constructed granulepos if the stored granulepos
+ * does not belong to this packet, or -1 if this codec does not have support
+ * for granulepos interpolation
+ * \param oggz An OGGZ handle
+ * \returns the granulepos of the \a current packet (if available)
+ */
+ogg_int64_t
+oggz_tell_granulepos (OGGZ * oggz);
+
 /**
  * Query the file offset in bytes corresponding to the data read.
  * \param oggz An OGGZ handle
@@ -113,11 +125,11 @@ ogg_int64_t oggz_seek_units (OGGZ * oggz, ogg_int64_t units, int whence);
  * \note When reading, the value returned by oggz_tell() reflects the
  * data offset of the start of the most recent packet processed, so that
  * when called from an OggzReadPacket callback it reflects the byte
- * offset of the start of the packet. As OGGZ may have internally read
+ * offset of the start of the packet. As Oggz may have internally read
  * ahead, this may differ from the current offset of the associated file
  * descriptor.
  */
-off_t oggz_tell (OGGZ * oggz);
+oggz_off_t oggz_tell (OGGZ * oggz);
 
 /**
  * Seek to a specific byte offset
@@ -126,7 +138,7 @@ off_t oggz_tell (OGGZ * oggz);
  * \param whence As defined in <stdio.h>: SEEK_SET, SEEK_CUR or SEEK_END
  * \returns the new file offset, or -1 on failure.
  */
-off_t oggz_seek (OGGZ * oggz, off_t offset, int whence);
+oggz_off_t oggz_seek (OGGZ * oggz, oggz_off_t offset, int whence);
 
 #ifdef _UNIMPLEMENTED
 long oggz_seek_packets (OGGZ * oggz, long serialno, long packets, int whence);
@@ -164,7 +176,7 @@ long oggz_seek_packets (OGGZ * oggz, long serialno, long packets, int whence);
  * \link basics Ogg Basics \endlink section).
  *
  * For data such as media, for which it is possible to provide a mapping
- * such as 'time', OGGZ can efficiently navigate through an Ogg stream
+ * such as 'time', Oggz can efficiently navigate through an Ogg stream
  * by use of an OggzMetric callback, thus allowing automatic seeking to
  * points in 'time'.
  *
@@ -306,7 +318,7 @@ int oggz_set_granulerate (OGGZ * oggz, long serialno,
  * This is the signature of a function to correlate Ogg streams.
  * If every position in an Ogg stream can be described by a metric (eg. time)
  * then define this function that returns some arbitrary unit value.
- * This is the normal use of OGGZ for media streams. The meaning of units is
+ * This is the normal use of Oggz for media streams. The meaning of units is
  * arbitrary, but must be consistent across all logical bitstreams; for
  * example a conversion of the time offset of a given packet into nanoseconds
  * or a similar stream-specific subdivision may be appropriate.
@@ -419,7 +431,7 @@ long oggz_seek_byorder (OGGZ * oggz, void * target);
 #endif /* _UNIMPLEMENTED */
 
 /**
- * Tell OGGZ to remember the given offset as the start of data.
+ * Tell Oggz to remember the given offset as the start of data.
  * This informs the seeking mechanism that when seeking back to unit 0,
  * go to the given offset, not to the start of the file, which is usually
  * codec headers.
@@ -431,7 +443,7 @@ long oggz_seek_byorder (OGGZ * oggz, void * target);
  * \param offset The offset of the start of data
  * \returns 0 on success, -1 on failure.
  */
-int oggz_set_data_start (OGGZ * oggz, off_t offset);
+int oggz_set_data_start (OGGZ * oggz, oggz_off_t offset);
 /** \}
  */
 
