@@ -101,6 +101,8 @@ oggz_new (int flags)
   oggz->order = NULL;
   oggz->order_user_data = NULL;
 
+  oggz->packet_buffer = oggz_dlist_new ();
+
   if (OGGZ_CONFIG_WRITE && (oggz->flags & OGGZ_WRITE)) {
     oggz_write_init (oggz);
   } else if (OGGZ_CONFIG_READ) {
@@ -167,12 +169,17 @@ oggz_stream_clear (void * data)
 {
   oggz_stream_t * stream = (oggz_stream_t *) data;
 
+  oggz_comments_free (stream);
+
   if (stream->ogg_stream.serialno != -1)
     ogg_stream_clear (&stream->ogg_stream);
 
   if (stream->metric_internal)
     oggz_free (stream->metric_user_data);
 
+  if (stream->calculate_data != NULL)
+    oggz_free (stream->calculate_data);
+  
   oggz_free (stream);
 
   return 0;
@@ -192,6 +199,9 @@ oggz_close (OGGZ * oggz)
   oggz_vector_foreach (oggz->streams, oggz_stream_clear);
   oggz_vector_delete (oggz->streams);
 
+  assert(oggz_dlist_is_empty(oggz->packet_buffer));
+  oggz_dlist_delete(oggz->packet_buffer);
+  
   if (oggz->metric_internal)
     oggz_free (oggz->metric_user_data);
 
@@ -211,7 +221,7 @@ oggz_close (OGGZ * oggz)
   return 0;
 }
 
-off_t
+oggz_off_t
 oggz_tell (OGGZ * oggz)
 {
   if (oggz == NULL) return OGGZ_ERR_BAD_OGGZ;
@@ -239,6 +249,22 @@ oggz_tell_units (OGGZ * oggz)
   }
 }
 
+ogg_int64_t
+oggz_tell_granulepos (OGGZ * oggz)
+{
+  if (oggz == NULL) return OGGZ_ERR_BAD_OGGZ;
+  
+  if (oggz->flags & OGGZ_WRITE) {
+    return OGGZ_ERR_INVALID;
+  }
+
+  if (OGGZ_CONFIG_READ) {
+    return oggz->x.reader.current_granulepos;
+  } else {
+    return OGGZ_ERR_DISABLED;
+  }
+}
+
 long
 oggz_run (OGGZ * oggz)
 {
@@ -280,9 +306,9 @@ oggz_find_stream (void * data, long serialno)
 oggz_stream_t *
 oggz_get_stream (OGGZ * oggz, long serialno)
 {
-  if (serialno < 0) return NULL;
+  if (serialno == -1) return NULL;
 
-  return oggz_vector_find (oggz->streams, oggz_find_stream, serialno);
+  return oggz_vector_find_with (oggz->streams, oggz_find_stream, serialno);
 }
 
 oggz_stream_t *
@@ -295,8 +321,10 @@ oggz_add_stream (OGGZ * oggz, long serialno)
 
   ogg_stream_init (&stream->ogg_stream, (int)serialno);
 
+  oggz_comments_init (stream);
+
   stream->content = OGGZ_CONTENT_UNKNOWN;
-  stream->nr_headers = 0;
+  stream->numheaders = 3; /* Default to 3 headers for Ogg logical bitstreams */
   stream->preroll = 0;
   stream->granulerate_n = 1;
   stream->granulerate_d = 1;
@@ -307,7 +335,7 @@ oggz_add_stream (OGGZ * oggz, long serialno)
   stream->b_o_s = 1;
   stream->e_o_s = 0;
   stream->granulepos = 0;
-  stream->packetno = -1; /* will be incremented on first write */
+  stream->packetno = -1; /* will be incremented on first read or write */
 
   stream->metric = NULL;
   stream->metric_user_data = NULL;
@@ -319,6 +347,8 @@ oggz_add_stream (OGGZ * oggz, long serialno)
   stream->read_page = NULL;
   stream->read_page_user_data = NULL;
 
+  stream->calculate_data = NULL;
+  
   oggz_vector_insert_p (oggz->streams, stream);
 
   return stream;
@@ -407,16 +437,40 @@ oggz_set_eos (OGGZ * oggz, long serialno)
   return 0;
 }
 
+int
+oggz_get_numtracks (OGGZ * oggz)
+{
+  if (oggz == NULL) return OGGZ_ERR_BAD_OGGZ;
+  return oggz_vector_size (oggz->streams);
+}
+
+/* Generate a pseudorandom serialno on request, ensuring that the number
+ * selected is not -1 or the serialno of an existing logical bitstream.
+ * NB. This inlines a simple linear congruential generator to avoid problems
+ * of portability of rand() vs. the POSIX random()/initstate()/getstate(), and
+ * in the case of rand() in order to avoid interfering with the random number
+ * sequence.
+ * Adapated from a patch by Erik de Castro Lopo, July 2007.
+ */
 long
 oggz_serialno_new (OGGZ * oggz)
 {
-  long serialno;
+  /* Ensure that the returned value is within the range of an int, so that
+   * it passes cleanly through ogg_stream_init(). In any case, the size of
+   * a serialno in the Ogg page header is 32 bits; it should never have been
+   * declared a long in ogg.h's ogg_packet in the first place. */
+  static ogg_int32_t serialno = 0;
+  int k;
+
+  if (serialno == 0) serialno = time(NULL);
 
   do {
-    serialno = oggz_random();
-  } while (oggz_get_stream (oggz, serialno) != NULL);
+    for (k = 0; k < 3 || serialno == 0; k++)
+      serialno = 11117 * serialno + 211231;
+  } while (serialno == -1 || oggz_get_stream (oggz, serialno) != NULL);
 
-  return serialno;
+  /* Cast the result back to a long for API consistency */
+  return (long)serialno;
 }
 
 /******** OggzMetric management ********/
