@@ -39,8 +39,9 @@
 #include "oggz_macros.h"
 
 typedef int (*OggzFunc) (void * data);
+typedef int (*OggzFunc1) (void * data, void * arg);
 typedef int (*OggzFindFunc) (void * data, long serialno);
-typedef int (*OggzCmpFunc) (void * a, void * b, void * user_data);
+typedef int (*OggzCmpFunc) (const void * a, const void * b, void * user_data);
 
 typedef struct _OggzVector OggzVector;
 
@@ -136,15 +137,49 @@ oggz_vector_nth_l (OggzVector * vector, int n)
 }
 
 void *
-oggz_vector_find (OggzVector * vector, OggzFindFunc func, long serialno)
+oggz_vector_find_p (OggzVector * vector, const void * data)
 {
-  void * data;
+  void * d;
+  int i;
+
+  if (vector->compare == NULL) return NULL;
+
+  for (i = 0; i < vector->nr_elements; i++) {
+    d = vector->data[i].p;
+    if (vector->compare (d, data, vector->compare_user_data))
+      return d;
+  }
+
+  return NULL;
+}
+
+int
+oggz_vector_find_index_p (OggzVector * vector, const void * data)
+{
+  void * d;
+  int i;
+
+  if (vector->compare == NULL) return -1;
+
+  for (i = 0; i < vector->nr_elements; i++) {
+    d = vector->data[i].p;
+    if (vector->compare (d, data, vector->compare_user_data))
+      return i;
+  }
+
+  return -1;
+}
+
+void *
+oggz_vector_find_with (OggzVector * vector, OggzFindFunc func, long serialno)
+{
+  void * d;
   int i;
 
   for (i = 0; i < vector->nr_elements; i++) {
-    data = vector->data[i].p;
-    if (func (data, serialno))
-      return data;
+    d = vector->data[i].p;
+    if (func (d, serialno))
+      return d;
   }
 
   return NULL;
@@ -162,6 +197,18 @@ oggz_vector_foreach (OggzVector * vector, OggzFunc func)
   return 0;
 }
 
+int
+oggz_vector_foreach1 (OggzVector * vector, OggzFunc1 func, void *arg)
+{
+  int i;
+
+  for (i = 0; i < vector->nr_elements; i++) {
+    func (vector->data[i].p, arg);
+  }
+
+  return 0;
+}
+
 static void
 _array_swap (oggz_data_t v[], int i, int j)
 {
