@@ -30,12 +30,20 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <getopt.h>
 #include <errno.h>
 
+#ifdef HAVE_INTTYPES_H
+#  include <inttypes.h>
+#else
+#  define PRId64 "I64d"
+#endif
+
 #include <oggz/oggz.h>
 
 #include "oggz_tools.h"
@@ -44,6 +52,8 @@
 
 #define SUBSECONDS 1000.0
 
+/* #define DEBUG */
+
 typedef ogg_int64_t timestamp_t;
 
 typedef struct _OVData {
@@ -173,7 +183,7 @@ read_page (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
   int ret = 0;
 
   if (ogg_page_bos ((ogg_page *)og)) {
-    content_type = ot_page_identify (og, NULL);
+    content_type = ot_page_identify (oggz, og, NULL);
 
     if (content_type) {
       if (!strcmp (content_type, "Theora")) {
@@ -190,7 +200,7 @@ read_page (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
 
   if(gpos != -1 && ogg_page_packets((ogg_page *)og) == 0) {
     ret = log_error ();
-    fprintf (stderr, "serialno %010ld: granulepos %lld on page with no completed packets, must be -1\n", serialno, gpos);
+    fprintf (stderr, "serialno %010ld: granulepos %" PRId64 " on page with no completed packets, must be -1\n", serialno, gpos);
   }
 
   return ret;
@@ -284,7 +294,7 @@ read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
   if ((feed_err = oggz_write_feed (ovdata->writer, op, serialno, flush, NULL)) != 0) {
     ret = log_error ();
     if (timestamp == -1.0) {
-      fprintf (stderr, "%ld", oggz_tell (oggz));
+      fprintf (stderr, "%" PRId64 , oggz_tell (oggz));
     } else {
       ot_fprint_time (stderr, (double)timestamp/SUBSECONDS);
     }
@@ -343,11 +353,18 @@ validate (char * filename)
   oggz_set_read_page (reader, -1, read_page, &ovdata);
 
   while (active && (n = oggz_read (reader, 1024)) != 0) {
+#ifdef DEBUG
+      fprintf (stderr, "validate: read %ld bytes\n", n);
+#endif
+    
     if (max_errors && nr_errors > max_errors) {
       fprintf (stderr,
 	       "oggz-validate: maximum error count reached, bailing out ...\n");
       active = 0;
     } else while ((nout = oggz_write_output (ovdata.writer, buf, n)) > 0) {
+#ifdef DEBUG
+      fprintf (stderr, "validate: wrote %ld bytes\n", nout);
+#endif
       bytes_written += nout;
     }
   }
