@@ -43,6 +43,12 @@
 #define snprintf _snprintf
 #endif  
 
+#ifdef HAVE_INTTYPES_H
+#  include <inttypes.h>
+#else
+#  define PRId64 "I64d"
+#endif
+
 static  ogg_uint32_t
 _le_32 (ogg_uint32_t i)
 {
@@ -115,13 +121,6 @@ _le_64 (ogg_int64_t l)
 
 typedef char * (* OTCodecInfoFunc) (unsigned char * data, long n);
 
-typedef struct {
-  const char *bos_str;
-  int bos_str_len;
-  const char *content_type;
-  OTCodecInfoFunc info_func;
-} OTCodecIdent;
-
 static char *
 ot_theora_info (unsigned char * data, long len)
 {
@@ -176,6 +175,53 @@ ot_speex_info (unsigned char * data, long len)
   return buf;
 }
 
+static char *
+ot_celt_info (unsigned char * data, long len)
+{
+  char * buf;
+
+  if (len < 56) return NULL;
+
+  buf = malloc (60);
+
+  snprintf (buf, 60,
+	    "\tAudio-Samplerate: %d Hz\n\tAudio-Channels: %d\n",
+	    INT32_LE_AT(&data[40]), INT32_LE_AT(&data[44]));
+
+  return buf;
+}
+
+static char *
+ot_flac_info (unsigned char * data, long len)
+{
+  char * buf;
+  int n;
+  int version_major, version_minor;
+  int samplerate;
+  int channels;
+
+  if (len < 30) return NULL;
+
+  buf = malloc (120);
+
+  version_major = data[5];
+  version_minor = data[6];
+
+  samplerate = (ogg_int64_t) (data[27] << 12) | (data[28] << 4) | 
+               ((data[29] >> 4)&0xf);
+  channels = 1 + ((data[29] >> 1)&0x7);
+
+  n = snprintf (buf, 120,
+	    "\tAudio-Samplerate: %d Hz\n\tAudio-Channels: %d\n",
+            samplerate, channels);
+
+  snprintf (buf+n, 120-n,
+            "\tFLAC-Ogg-Mapping-Version: %d.%d\n",
+            version_major, version_minor);
+
+  return buf;
+}
+
 static char *
 ot_oggpcm2_info (unsigned char * data, long len)
 {
@@ -196,57 +242,74 @@ static char *
 ot_skeleton_info (unsigned char * data, long len)
 {
   char * buf;
+  double pres_n, pres_d, pres;
+  double base_n, base_d, base;
 
-  if (len < 64) return NULL;
+  if (len < 64L) return NULL;
 
   buf = malloc (60);
 
+  pres_n = (double)INT64_LE_AT(&data[12]);
+  pres_d = (double)INT64_LE_AT(&data[20]);
+  if (pres_d != 0.0) {
+    pres = pres_n / pres_d;
+  } else {
+    pres = 0.0;
+  }
+
+  base_n = (double)INT64_LE_AT(&data[12]);
+  base_d = (double)INT64_LE_AT(&data[20]);
+  if (base_d != 0.0) {
+    base = base_n / base_d;
+  } else {
+    base = 0.0;
+  }
+
   snprintf (buf, 60,
-	    "\tPresentation-Time: %.3f\n\tBasetime: %.3f\n",
-	    (double)INT64_LE_AT(&data[12]) / (double)INT64_LE_AT(&data[20]),
-	    (double)INT64_LE_AT(&data[28]) / (double)INT64_LE_AT(&data[36]));
+	    "\tPresentation-Time: %.3f\n\tBasetime: %.3f\n", pres, base);
 
   return buf;
 }
 
-static const OTCodecIdent codec_ident[] = {
-  {"\200theora", 7, "Theora", ot_theora_info},
-  {"\001vorbis", 7, "Vorbis", ot_vorbis_info},
-  {"Speex", 5, "Speex", ot_speex_info},
-  {"PCM     ", 8, "PCM", ot_oggpcm2_info},
-  {"CMML\0\0\0\0", 8, "CMML", NULL},
-  {"Annodex", 8, "Annodex", NULL},
-  {"fishead", 8, "Skeleton", ot_skeleton_info},
-  {NULL}
+static const OTCodecInfoFunc codec_ident[] = {
+  ot_theora_info, 
+  ot_vorbis_info,
+  ot_speex_info,
+  ot_oggpcm2_info,
+  NULL,             /* CMML */
+  NULL,             /* ANNODEX */
+  ot_skeleton_info,
+  NULL,             /* FLAC0 */
+  ot_flac_info,     /* FLAC */
+  NULL,             /* ANXDATA */
+  ot_celt_info,     /* CELT */
+  NULL              /* UNKOWN */
 };
 
 const char *
-ot_page_identify (const ogg_page * og, char ** info)
+ot_page_identify (OGGZ *oggz, const ogg_page * og, char ** info)
 {
   const char * ret = NULL;
-  int i;
+  int serial_no;
+  int content;
 
-  /* try to identify stream codec name by looking at the first bytes of the
-   * first packet */
-  for (i = 0;; i++) {
-    const OTCodecIdent *ident = &codec_ident[i];
-    
-    if (ident->bos_str == NULL) {
-      ret = NULL;
-      break;
-    }
+  /*
+   * identify stream content using oggz_stream_get_content, identify
+   * stream content name using oggz_stream_get_content_type
+   */
+  
+  serial_no = ogg_page_serialno((ogg_page *)og);
+  
+  content = oggz_stream_get_content(oggz, serial_no);
+  if (content == OGGZ_ERR_BAD_SERIALNO) return NULL;
 
-    if (og->body_len >= ident->bos_str_len &&
-	memcmp (og->body, ident->bos_str, ident->bos_str_len) == 0) {
-      ret = ident->content_type;
-      if (info) {
-	if (ident->info_func) {
-	  *info = ident->info_func (og->body, og->body_len);
-	} else {
-	  *info = NULL;
-	}
-      }
-      break;
+  ret = oggz_stream_get_content_type(oggz, serial_no);
+
+  if (info != NULL)
+  {
+    if (codec_ident[content] != NULL)
+    {
+      *info = codec_ident[content](og->body, og->body_len);
     }
   }
 
@@ -315,6 +378,25 @@ ot_fprint_time (FILE * stream, double seconds)
   return fprintf (stream, "%s%02d:%02d:%06.3f", sign, hrs, min, sec);
 }
 
+int
+ot_fprint_granulepos (FILE * stream, OGGZ * oggz, long serialno,
+                      ogg_int64_t granulepos)
+{
+  int ret, granuleshift = oggz_get_granuleshift (oggz, serialno);
+
+  if (granuleshift < 1) {
+    ret = fprintf (stream, "%" PRId64, granulepos);
+  } else {
+    ogg_int64_t iframe, pframe;
+    iframe = granulepos >> granuleshift;
+    pframe = granulepos - (iframe << granuleshift);
+
+    ret = fprintf (stream, "%" PRId64 "|%" PRId64, iframe, pframe);
+  }
+
+  return ret;
+}
+
 void
 ot_init (void)
 {
