@@ -35,6 +35,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifndef WIN32
+#include <strings.h>
+#endif
 #include <ctype.h>
 
 #ifdef HAVE_INTTYPES_H
@@ -230,28 +233,20 @@ bin_dump (unsigned char * buf, long n, int dump_char)
   }
 }
 
-/* FIXME: on Mac OS X, off_t is 64-bits.  Obviously we want a nicer
- * way to do it than this, but a quick fix is a good fix */
-#ifdef __APPLE__
-#  define PRI_off_t "q"
-#else
-#  define PRI_off_t "l"
-#endif
-
 static int
 read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 {
   ODData * oddata = (ODData *) user_data;
   ogg_int64_t units;
   double time_offset;
-  int granuleshift;
+  ogg_int64_t calced_gp = oggz_tell_granulepos (oggz);
 
   if (oddata->hide_offset) {
     fprintf (outfile, "oOo");
   } else {
     units = oggz_tell_units (oggz);
     if (units == -1) {
-      fprintf (outfile, "%08" PRI_off_t "x", oggz_tell (oggz));
+      fprintf (outfile, "%" PRId64 "x", oggz_tell (oggz));
     } else {
       time_offset = (double)units / 1000.0;
       ot_fprint_time (outfile, time_offset);
@@ -262,22 +257,22 @@ read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 	   oddata->hide_serialno ? -1 : serialno);
 
   if (oddata->hide_granulepos) {
-    fprintf (outfile, "granulepos gGg,");
+    fprintf (outfile, "granulepos gGg");
   } else {
-    granuleshift = oggz_get_granuleshift (oggz, serialno);
-    if (granuleshift < 1) {
-      fprintf (outfile, "granulepos %" PRId64 ",", op->granulepos);
+    if (op->granulepos == -1) {
+      fprintf (outfile, "calc. gpos ");
     } else {
-      ogg_int64_t iframe, pframe;
-      iframe = op->granulepos >> granuleshift;
-      pframe = op->granulepos - (iframe << granuleshift);
+      fprintf (outfile, "granulepos ");
+    }
+    ot_fprint_granulepos (outfile, oggz, serialno, calced_gp);
 
-      fprintf (outfile, "granulepos %" PRId64 "|%" PRId64 ",",
-	       iframe, pframe);
+    if (op->granulepos != -1 && op->granulepos != calced_gp) {
+      fprintf (outfile, " ERR: file gp ");
+      ot_fprint_granulepos (outfile, oggz, serialno, op->granulepos);
     }
   }
 
-  fprintf (outfile, " packetno %" PRId64,
+  fprintf (outfile, ", packetno %" PRId64,
 	   oddata->hide_packetno ? -1 : op->packetno);
 
   if (op->b_o_s) {
@@ -311,7 +306,7 @@ filter_page (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
   int i, n;
 
   if (ogg_page_bos ((ogg_page *)og)) {
-    ident = ot_page_identify (og, NULL);
+    ident = ot_page_identify (oggz, og, NULL);
     if (ident) {
       n = oggz_table_size (oddata->content_types_table);
       for (i = 0; i < n; i++) {
@@ -337,12 +332,34 @@ read_new_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
   }
 }
 
+static void
+revert_packet (OGGZ * oggz, ogg_packet * op, long serialno, int flush)
+{
+  unsigned char buf[1024];
+  long n;
+  int ret;
+
+#ifdef DEBUG
+  printf ("feeding packet (%010ld) %ld bytes %s, %s\n",
+          current_serialno, op->bytes,
+          op->b_o_s ? "bos" : "not bos",
+          op->e_o_s ? "eos" : "not eos");
+#endif
+  if ((ret = oggz_write_feed (oggz, op, serialno, flush, NULL)) != 0) {
+    fprintf (stderr, "%s: oggz_write_feed error %d\n", progname, ret);
+  }
+
+  while ((n = oggz_write_output (oggz, buf, 1024)) > 0) {
+    fwrite (buf, 1, n, outfile);
+  }
+}
+
 static void
 revert_file (char * infilename)
 {
   OGGZ * oggz;
   FILE * infile;
-  char line[80];
+  char line[120];
   int hh, mm, ss;
   unsigned int offset;
   long current_serialno = -1, serialno;
@@ -355,10 +372,8 @@ revert_file (char * infilename)
   unsigned char * packet = NULL;
   long max_bytes = 0;
 
-  unsigned char buf[1024];
   ogg_packet op;
   int flush = 1;
-  long n;
   char c;
 
   if (strcmp (infilename, "-") == 0) {
@@ -369,7 +384,7 @@ revert_file (char * infilename)
 
   oggz = oggz_new (OGGZ_WRITE|OGGZ_NONSTRICT|OGGZ_AUTO);
 
-  while (fgets (line, 80, infile)) {
+  while (fgets (line, 120, infile)) {
     line_offset = 0;
 
     /* Skip time offsets, OR ensure line_offset is 0 */
@@ -377,12 +392,12 @@ revert_file (char * infilename)
       line_offset = 0;
 
     is_packetinfo = 0;
-    if (sscanf (&line[line_offset], "%x: serialno %ld, granulepos %lld, packetno %lld%n",
+    if (sscanf (&line[line_offset], "%x: serialno %ld, granulepos %" PRId64 ", packetno %" PRId64 "%n",
 		&offset, &serialno, &granulepos, &packetno,
 		&line_offset) >= 4) {
       is_packetinfo = 1;
     } else {
-      if (sscanf (&line[line_offset], "%x: serialno %ld, granulepos %lld|%lld, packetno %lld%n",
+      if (sscanf (&line[line_offset], "%x: serialno %ld, granulepos %" PRId64 "%" PRId64 ", packetno %" PRId64 "%n",
 		  &offset, &serialno, &iframe, &pframe, &packetno,
 		  &line_offset) >= 5) {
 	int granuleshift = oggz_get_granuleshift (oggz, serialno);
@@ -394,20 +409,7 @@ revert_file (char * infilename)
     if (is_packetinfo) {
       /* flush any existing packets */
       if (current_serialno != -1) {
-	int ret;
-
-#ifdef DEBUG
-	printf ("feeding packet (%010ld) %ld bytes %s\n",
-		current_serialno, op.bytes,
-		op.b_o_s ? "bos" : "not bos");
-#endif
-	if ((ret = oggz_write_feed (oggz, &op, current_serialno, flush, NULL)) != 0) {
-	  fprintf (stderr, "%s: oggz_write_feed error %d\n", progname, ret);
-	}
-
-	while ((n = oggz_write_output (oggz, buf, 1024)) > 0) {
-	  fwrite (buf, 1, n, outfile);
-	}
+        revert_packet (oggz, &op, current_serialno, flush);
       }
 
       /* Start new packet */
@@ -440,6 +442,8 @@ revert_file (char * infilename)
 
       if (current_serialno != -1 &&
 	  sscanf (line, "%x:%n", &offset, &line_offset) >= 1) {
+        /* NUL-terminate after hex data: don't scan char representation */
+        line[50] = '\0';
 	while (nread < 16 &&
 	       (sscanf (&line[line_offset], "%2x%n", &val, &consumed) > 0)) {
 	  op.bytes++;
@@ -476,6 +480,11 @@ revert_file (char * infilename)
     }
   }
 
+  /* flush any existing packets */
+  if (current_serialno != -1) {
+    revert_packet (oggz, &op, current_serialno, flush);
+  }
+
   fclose (infile);
 }
 
